@@ -1,31 +1,43 @@
+# -*- coding: utf-8 -*-
 import base64
 import hashlib
 
 import six
 import xml
+
+from ks3.bucketCrossReplicate import BucketCrossReplicate
+from ks3.bucketLifecycle import BucketLifecycle
+from ks3.objectTagging import ObjectTagging
+
 try:
     import urllib.parse as parse  # for Python 3
 except ImportError:
     import urllib as parse  # for Python 2
 
+from dateutil import parser
 import ks3.utils
 from ks3 import handler
 from ks3.acl import Policy, CannedACLStrings
 from ks3.bucketlistresultset import BucketListResultSet
 from ks3.bucketlistresultset import MultiPartUploadListResultSet
+from ks3.bucketlistresultset import VersionedBucketListResultSet
 from ks3.bucketlogging import BucketLogging
 from ks3.exception import S3ResponseError
 from ks3.key import Key
 from ks3.multipart import MultiPartUpload, CompleteMultiPartUpload
 from ks3.prefix import Prefix
 from ks3.resultset import ResultSet
+from ks3.bucketQuota import BucketQuota
+from ks3.bucketVersioning import BucketVersioningConfig
+from ks3.deletemarker import DeleteMarker
+
 try:
     from ks3.encryption import Crypts
 except:
     pass
 
-class Bucket(object):
 
+class Bucket(object):
     def __init__(self, connection=None, name=None):
         self.connection = connection
         self.name = name
@@ -50,7 +62,7 @@ class Bucket(object):
         else:
             setattr(self, name, value)
 
-    def new_key(self, key_name=None):       
+    def new_key(self, key_name=None):
         """
         Creates a new key 
         
@@ -61,10 +73,11 @@ class Bucket(object):
         :returns: An instance of the newly created key object
         """
         if not key_name:
-            raise ValueError('Empty key names are not allowed')                                                  
+            raise ValueError('Empty key names are not allowed')
         return Key(self, key_name)
 
-    def copy_key(self, new_key_name, src_bucket_name, src_key_name, headers=None, query_args=None, encrypt_key=False):
+    def copy_key(self, new_key_name, src_bucket_name, src_key_name, src_version_id=None, headers=None, query_args=None,
+                 encrypt_key=False):
         """
         Create a new key in the bucket by copying another existing key.
         :param new_key_name: The name of the new key
@@ -88,6 +101,8 @@ class Bucket(object):
             headers[provider.server_side_encryption_header] = 'AES256'
         src = '/%s/%s' % (src_bucket_name, parse.quote_plus(src_key_name.encode('utf-8')))
         src = src.replace('//', '/%2F')
+        if src_version_id:
+            src += '?versionId=%s' % src_version_id
         headers[provider.copy_source_header] = str(src)
         response = self.connection.make_request('PUT', self.name, new_key_name,
                                                 headers=headers,
@@ -106,15 +121,13 @@ class Bucket(object):
             raise provider.storage_response_error(response.status,
                                                   response.reason, body)
 
-
-
     def generate_url(self, expires_in, method='GET', headers=None,
                      force_http=False, response_headers=None,
                      expires_in_absolute=False):
         return self.connection.generate_url(expires_in, method, self.name,
                                             headers=headers,
                                             force_http=force_http,
-                                            response_headers=response_headers,                                   
+                                            response_headers=response_headers,
                                             expires_in_absolute=expires_in_absolute)
 
     # def delete_keys(self, keys, quiet=False, mfa_token=None, headers=None):
@@ -200,15 +213,15 @@ class Bucket(object):
         Check to see if a particular key exists within the bucket.
         """
         if validate is False:
-           # if headers or version_id or response_headers:
-           #     raise BotoClientError(
-           #         "When providing 'validate=False', no other params " + \
-           #         "are allowed."
-           #     )
+            # if headers or version_id or response_headers:
+            #     raise BotoClientError(
+            #         "When providing 'validate=False', no other params " + \
+            #         "are allowed."
+            #     )
 
-           # This leans on the default behavior of ``new_key`` (not hitting
-           # the service). If that changes, that behavior should migrate here.
-           return self.new_key(key_name)
+            # This leans on the default behavior of ``new_key`` (not hitting
+            # the service). If that changes, that behavior should migrate here.
+            return self.new_key(key_name)
 
         query_args_l = {}
         if version_id:
@@ -231,7 +244,7 @@ class Bucket(object):
         if response.status // 100 == 2:
             k = Key(self)
             # provider = self.connection.provider
-            #k.metadata = boto.utils.get_aws_metadata(response.msg, provider)
+            # k.metadata = boto.utils.get_aws_metadata(response.msg, provider)
             for field in Key.base_fields:
                 k.__dict__[field.lower().replace('-', '_')] = \
                     response.getheader(field)
@@ -251,6 +264,9 @@ class Bucket(object):
             k.handle_addl_headers(response.getheaders())
             k.handle_user_metas(response)
             k.handle_storage_class(response)
+            k.handle_tagging_count(response)
+            k.handle_object_type(response)
+            k.handle_next_position(response)
             return k, response
         else:
             if response.status == 404:
@@ -258,18 +274,68 @@ class Bucket(object):
             else:
                 raise S3ResponseError(response.status, response.reason, None)
 
-    def list(self, prefix=None, delimiter=None, marker=None):
+    def list(self, prefix=None, delimiter='/', marker=None, max_keys=None):
         """
         List object keys within specified bucket.
         """
-        return BucketListResultSet(self, prefix, delimiter, marker)
+        return BucketListResultSet(self, prefix, delimiter, marker, max_keys)
+
+    def listObjects(self, prefix=None, delimiter=None, marker=None, startTime=None, endTime=None,
+                    filename=None):
+
+        keys = self.list(prefix, delimiter, marker)
+        keyLists = list()
+        fd = None
+        if filename is not None:
+            fd = open(filename, "a")  # 利用追加模式,参数从w替换为a即可
+            fd.seek(0)
+        for k in keys:
+            datetime = int(round(parser.parse(k.last_modified).timestamp()))
+            if startTime is None and endTime is not None:
+                if (datetime >= endTime):
+                    keyLists.append(k.name)
+                    if fd:
+                        fd.write("{}\n".format(k.name))
+            elif endTime is None and startTime is not None:
+                if (datetime <= startTime):
+                    keyLists.add(k.name)
+                    if fd:
+                        fd.write("{}\n".format(k.name))
+            elif endTime is not None and startTime is not None:
+                if (startTime <= datetime & datetime <= endTime):
+                    keyLists.add(k.name)
+                    if fd:
+                        fd.write("{}\n".format(k.name))
+            elif endTime is None and startTime is None:
+                if fd:
+                    fd.write("{}\n".format(k.name))
+
+        if fd:
+            fd.flush()
+            fd.close()
+        return keyLists
+
+    def list_versions(self, prefix='', delimiter='', key_marker='',
+                      version_id_marker='', headers=None, encoding_type=None):
+        return VersionedBucketListResultSet(self, prefix, delimiter,
+                                            key_marker, version_id_marker,
+                                            headers,
+                                            encoding_type=encoding_type)
 
     def get_all_keys(self, **params):
         """
         A lower-level method for listing contents of a bucket.
         """
-        return self._get_all([('Contents', Key), ('CommonPrefixes', Prefix)],
-                             '', **params)
+        return self._get_all([('Contents', Key), ('CommonPrefixes', Prefix)], '', **params)
+
+    def get_all_versions(self, headers=None, **params):
+        """
+        A lower-level method for listing contents of a bucket.
+        """
+        return self._get_all([('Version', Key),
+                              ('CommonPrefixes', Prefix),
+                              ('DeleteMarker', DeleteMarker)],
+                             'versions', headers, **params)
 
     def _get_all(self, element_map, initial_query_string='',
                  headers=None, **params):
@@ -300,7 +366,7 @@ class Bucket(object):
         for key, value in sorted(list(params.items()), key=lambda x: x[0]):
             if value is None:
                 continue
-            key = key.replace('_', '-') 
+            key = key.replace('_', '-')
             if key == 'maxkeys':
                 key = 'max-keys'
             if not isinstance(value, six.string_types + (six.binary_type,)):
@@ -337,7 +403,6 @@ class Bucket(object):
             self.set_canned_acl(acl_or_str, key_name,
                                 headers, version_id)
 
-
     def set_canned_acl(self, acl_str, key_name='', headers=None,
                        version_id=None):
         assert acl_str in CannedACLStrings
@@ -351,7 +416,7 @@ class Bucket(object):
         if version_id:
             query_args += '&versionId=%s' % version_id
         response = self.connection.make_request('PUT', self.name, key_name,
-                headers=headers, query_args=query_args)
+                                                headers=headers, query_args=query_args)
         body = response.read()
         if response.status != 200:
             raise S3ResponseError(response.status, response.reason, body)
@@ -374,7 +439,6 @@ class Bucket(object):
         else:
             raise S3ResponseError(response.status, response.reason, body)
 
-
     def enable_logging(self, target_bucket, target_prefix='',
                        grants=None, headers=None):
         """
@@ -396,11 +460,11 @@ class Bucket(object):
         """
         if isinstance(target_bucket, Bucket):
             target_bucket = target_bucket.name
-        blogging = BucketLogging(target=target_bucket, prefix=target_prefix,
+        blogging = BucketLogging(target=target_bucket, target_prefix=target_prefix,
                                  grants=grants)
-        return self.set_xml_logging(blogging.to_xml(), headers=headers)
+        return self.set_bucket_logging(blogging.to_xml(), headers=headers)
 
-    def set_xml_logging(self, logging_str, headers=None):
+    def set_bucket_logging(self, logging_str, headers=None):
         """
         Set logging on a bucket directly to the given xml string.
 
@@ -413,11 +477,17 @@ class Bucket(object):
         :rtype: bool
         :return: True if ok or raises an exception.
         """
+
+        if headers is None:
+            headers = {}
         body = logging_str
         if not isinstance(body, bytes):
             body = body.encode('utf-8')
+            # md5 = ks3.utils.compute_base64_md5_digest(body)
+            # headers['Content-MD5'] = md5
+            headers['content-type'] = 'application/xml'
         response = self.connection.make_request('PUT', self.name, data=body,
-                query_args='logging', headers=headers)
+                                                query_args='logging', headers=headers)
         body = response.read()
         if response.status == 200:
             return True
@@ -432,17 +502,17 @@ class Bucket(object):
         :return: True if ok or raises an exception.
         """
         blogging = BucketLogging()
-        return self.set_xml_logging(blogging.to_xml(), headers=headers)
+        return self.set_bucket_logging(blogging.to_xml(), headers=headers)
 
-    def get_logging_status(self, headers=None):
+    def get_bucket_logging(self, headers=None):
         """
-        Get the logging status for this bucket.
+        Get the logging for this bucket.
 
-        :rtype: :class:`boto.s3.bucketlogging.BucketLogging`
+        :rtype: :class:`ks3.bucketlogging.BucketLogging`
         :return: A BucketLogging object for this bucket.
         """
         response = self.connection.make_request('GET', self.name,
-                query_args='logging', headers=headers)
+                                                query_args='logging', headers=headers)
         body = response.read()
         if response.status == 200:
             blogging = BucketLogging()
@@ -531,7 +601,7 @@ class Bucket(object):
         A lower-level, version-aware method for listing active
         MultiPart uploads for a bucket.
         """
-        #self.validate_kwarg_names(params, ['max_uploads', 'key_marker',
+        # self.validate_kwarg_names(params, ['max_uploads', 'key_marker',
         #                                   'upload_id_marker', 'encoding_type',
         #                                   'delimiter', 'prefix'])
         return self._get_all([('Upload', MultiPartUpload),
@@ -560,14 +630,14 @@ class Bucket(object):
             storage_class_header = provider.storage_class_header
             if storage_class_header:
                 headers[storage_class_header] = 'REDUCED_REDUNDANCY'
-            # TODO: what if the provider doesn't support reduced redundancy?
+                # TODO: what if the provider doesn't support reduced redundancy?
         if encrypt_key:
             headers[provider.server_side_encryption_header] = 'AES256'
         if metadata is None:
             metadata = {}
 
         headers = ks3.utils.merge_meta(headers, metadata,
-                self.connection.provider)
+                                       self.connection.provider)
         if self.connection.local_encrypt:
             crypts = Crypts(self.connection.key)
             crypts.calc_md5 = calc_encrypt_md5
@@ -577,8 +647,8 @@ class Bucket(object):
             headers["x-kss-meta-key"] = base64.b64encode(md5_generator.hexdigest().encode()).decode()
             headers["x-kss-meta-iv"] = base64.b64encode(crypts.first_iv).decode()
             response = self.connection.make_request('POST', self.name, key_name,
-                                                query_args=query_args,
-                                                headers=headers)
+                                                    query_args=query_args,
+                                                    headers=headers)
         else:
             response = self.connection.make_request('POST', self.name, key_name,
                                                     query_args=query_args,
@@ -651,6 +721,268 @@ class Bucket(object):
             raise self.connection.provider.storage_response_error(
                 response.status, response.reason, body)
 
+    def set_adp(self, key_name, headers):
+        query_args = 'adp'
+        response = self.connection.make_request('PUT', self.name, key_name,
+                                                headers=headers, query_args=query_args)
+        body = response.read()
+        if response.status != 200:
+            raise S3ResponseError(response.status, response.reason, body)
+        task_id = response.getheader("TaskID")
+        return task_id
+
+    def set_bucket_quota(self, headers=None, quota=0):
+        """
+        :param quota: bucket quota default 0 not limit
+        :param headers: custom header
+        :return: True is ok or raises an exception.
+        """
+        bucketQuota = BucketQuota(quota)
+        quota_to_xml = bucketQuota.to_xml()
+        if not isinstance(quota_to_xml, bytes):
+            quota_to_xml = quota_to_xml.encode('utf-8')
+        if headers is None:
+            headers = {}
+        headers['Content-Type'] = 'text/xml'
+        response = self.connection.make_request('PUT', self.name, data=quota_to_xml,
+                                                query_args='quota', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_quota(self, headers=None):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='quota', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            quota = BucketQuota()
+            h = handler.XmlHandler(quota, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return quota
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_policy(self, policy, headers=None):
+        response = self.connection.make_request('PUT', self.name, data=policy,
+                                                query_args='policy', headers=headers)
+        body = response.read()
+        if response.status == 204:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_policy(self, headers=None):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='policy', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return body
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_bucket_policy(self, headers=None):
+        response = self.connection.make_request('DELETE', self.name,
+                                                query_args='policy', headers=headers)
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_crr(self, targetBucket, deleteMarkerStatus=None, prefix=None, region=None, headers=None):
+        replicate = BucketCrossReplicate(targetBucket, deleteMarkerStatus, prefix, region)
+        if headers is None:
+            headers = {}
+        headers['Content-Type'] = 'text/xml'
+        replicate_xml = replicate.to_xml()
+        if not isinstance(replicate_xml, bytes):
+            replicate_xml = replicate_xml.encode('utf-8')
+        md5 = ks3.utils.compute_base64_md5_digest(replicate_xml)
+        headers['Content-MD5'] = md5
+        response = self.connection.make_request('PUT', self.name, data=replicate_xml,
+                                                query_args='crr', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_crr(self, headers=None):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='crr', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            replicate = BucketCrossReplicate()
+            h = handler.XmlHandler(replicate, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return replicate
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_bucket_crr(self, headers=None):
+        response = self.connection.make_request('DELETE', self.name,
+                                                query_args='crr', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def restore_object(self, object_key_name, headers=None):
+        response = self.connection.make_request('POST', self.name, object_key_name,
+                                                query_args='restore', headers=headers)
+        body = response.read()
+        if response.status == 200 or response.status == 202:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_object_tagging(self, object_key_name, tagging_set, version_id=None, headers=None):
+        object_tagging = ObjectTagging(tagging_set)
+        if headers is None:
+            headers = {}
+        headers['Content-Type'] = 'text/xml'
+        object_tagging_xml = object_tagging.to_xml()
+        if not isinstance(object_tagging_xml, bytes):
+            object_tagging_xml = object_tagging_xml.encode('utf-8')
+
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.connection.make_request('PUT', self.name, object_key_name, data=object_tagging_xml,
+                                                query_args=query_args, headers=headers)
+
+        body = response.read()
+        if response.status == 200:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_object_tagging(self, object_key_name, version_id=None, headers=None):
+
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.connection.make_request('GET', self.name, object_key_name,
+                                                query_args=query_args, headers=headers)
+        body = response.read()
+        if response.status == 200:
+            objectTagging = ObjectTagging()
+            h = handler.XmlHandler(objectTagging, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return objectTagging
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_object_tagging(self, object_key_name, version_id=None, headers=None):
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.connection.make_request('DELETE', self.name, object_key_name,
+                                                query_args=query_args, headers=headers)
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def fetch_object(self, object_key_name, source_url=None, callback_url=None, headers=None):
+        if headers is None:
+            headers = {}
+        if source_url is not None:
+            headers['x-kss-sourceurl'] = source_url
+        if callback_url is not None:
+            headers['x-kss-callbackurl'] = callback_url
+
+        response = self.connection.make_request('PUT', self.name, object_key_name,
+                                                query_args='fetch', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_lifecycle(self, bucketLifecycle, headers=None):
+        if headers is None:
+            headers = {}
+        bucketLifecycle_xml = bucketLifecycle.to_xml()
+        if not isinstance(bucketLifecycle_xml, bytes):
+            bucketLifecycle_xml = bucketLifecycle_xml.encode('utf-8')
+        md5 = ks3.utils.compute_base64_md5_digest(bucketLifecycle_xml)
+        headers['Content-MD5'] = md5
+        headers['content-type'] = 'application/xml'
+        response = self.connection.make_request('PUT', self.name, data=bucketLifecycle_xml,
+                                                query_args='lifecycle', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_lifecycle(self, headers=None):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='lifecycle', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            lifecycle = BucketLifecycle()
+            h = handler.XmlHandler(lifecycle, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return lifecycle
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    # delete bucket lifecycle
+    def delete_bucket_lifecycle(self, headers=None):
+        response = self.connection.make_request('DELETE', self.name,
+                                                query_args='lifecycle', headers=headers)
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_versioning(self, version_configuration, headers=None):
+        if headers is None:
+            headers = {}
+        version_configuration_xml = version_configuration.to_xml()
+        if not isinstance(version_configuration_xml, bytes):
+            version_configuration_xml = version_configuration_xml.encode('utf-8')
+        md5 = ks3.utils.compute_base64_md5_digest(version_configuration_xml)
+        headers['Content-MD5'] = md5
+        headers['content-type'] = 'application/xml'
+        response = self.connection.make_request('PUT', self.name, data=version_configuration_xml,
+                                                query_args='versioning',
+                                                headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_versioning(self, headers=None):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='versioning', headers=headers)
+        body = response.read()
+        if response.status == 200:
+            versioningConfig = BucketVersioningConfig()
+            h = handler.XmlHandler(versioningConfig, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return versioningConfig
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
 
 class BucketLocation(object):
     def __init__(self):
@@ -661,4 +993,4 @@ class BucketLocation(object):
 
     def endElement(self, name, current_text, connection):
         if name == 'LocationConstraint':
-            self.location = current_text
+            self.location = current_text
\ No newline at end of file
