@@ -5,7 +5,7 @@
 #  code either by itself or as incorporated into your code; provided that
 #  you do not remove any proprietary notices. Your use of this software
 #  code is at your own risk.
-
+import json
 import os
 import time
 import xml.sax
@@ -25,6 +25,7 @@ try:
 except ImportError:
     import urllib as parse  # for Python 2
 
+
 def check_lowercase_bucketname(n):
     """
     Bucket names must not contain uppercase characters. We check for
@@ -45,8 +46,8 @@ def check_lowercase_bucketname(n):
     """
     if not (n + 'a').islower():
         raise KS3ClientError("Bucket names cannot contain upper-case " \
-            "characters when using either the sub-domain or virtual " \
-            "hosting calling format.")
+                             "characters when using either the sub-domain or virtual " \
+                             "hosting calling format.")
     return True
 
 
@@ -99,12 +100,12 @@ class SubdomainCallingFormat(_CallingFormat):
 
 
 class Connection(object):
-
     QueryString = 'Signature=%s&Expires=%d&KSSAccessKeyId=%s'
 
     def __init__(self, access_key_id, access_key_secret, host="",
-            port=80, provider='kss', security_token=None, profile_name=None, path='/',
-            is_secure=False, debug=0, calling_format=SubdomainCallingFormat, domain_mode=False, local_encrypt=False, local_key_path=""):
+                 port=80, provider='kss', security_token=None, profile_name=None, path='/',
+                 is_secure=False, debug=0, calling_format=SubdomainCallingFormat, domain_mode=False,
+                 local_encrypt=False, local_key_path="", timeout=10):
         """
         :param access_key_id: 金山云提供的ACCESS KEY ID
         :param access_key_secret: 金山云提供的SECRET KEY ID
@@ -125,11 +126,12 @@ class Connection(object):
         self.domain_mode = domain_mode
         self.local_encrypt = local_encrypt
         self.key = ""
+        self.timeout = timeout
         if (self.is_secure):
             self.protocol = 'https'
             if self.port == 80:
                 self.port = 443
-        else:   
+        else:
             self.protocol = 'http'
 
         if isinstance(provider, Provider):
@@ -162,21 +164,21 @@ class Connection(object):
             content = ak_file.read()
             assert len(content.strip()) == 16, "The key's length should be 16"
             self.key = content.strip()
-        
 
     def make_request(self, method, bucket="", key="", data="",
-            headers=None, query_args=None, metadata=None):
+                     headers=None, query_args=None, metadata=None, timeout=10):
         if not headers:
             headers = {}
         if not query_args:
             query_args = {}
         if not metadata:
             metadata = {}
-
+        timeout = self.timeout
         resp = make_request(self.host, self.port, self.access_key_id,
                             self.access_key_secret, method, bucket, key,
-                            query_args, headers, data, metadata, is_secure=self.is_secure, domain_mode=self.domain_mode)
-        
+                            query_args, headers, data, metadata, is_secure=self.is_secure, domain_mode=self.domain_mode,
+                            timeout=timeout)
+
         return resp
 
     def get_all_buckets(self, headers=None):
@@ -218,9 +220,9 @@ class Connection(object):
             data = ''
         else:
             data = '<CreateBucketConfiguration><LocationConstraint>' + \
-                    location + '</LocationConstraint></CreateBucketConfiguration>'
+                   location + '</LocationConstraint></CreateBucketConfiguration>'
         response = self.make_request('PUT', bucket_name, headers=headers,
-                data=data)
+                                     data=data)
         body = response.read()
         if response.status == 409:
             raise S3CreateError(response.status, response.reason, body)
@@ -241,85 +243,6 @@ class Connection(object):
         if response.status != 204:
             raise S3ResponseError(response.status, response.reason, body)
 
-    # def set_xml_logging(self, logging_str, headers=None):
-    #     """
-    #     Set logging on a bucket directly to the given xml string.
-    #
-    #     :type logging_str: unicode string
-    #     :param logging_str: The XML for the bucketloggingstatus which
-    #         will be set.  The string will be converted to utf-8 before
-    #         it is sent.  Usually, you will obtain this XML from the
-    #         BucketLogging object.
-    #
-    #     :rtype: bool
-    #     :return: True if ok or raises an exception.
-    #     """
-    #     body = logging_str
-    #     if not isinstance(body, bytes):
-    #         body = body.encode('utf-8')
-    #     response = self.connection.make_request('PUT', self.name, data=body,
-    #             query_args={'logging':''}, headers=headers)
-    #     body = response.read()
-    #     if response.status == 200:
-    #         return True
-    #     else:
-    #         raise S3ResponseError(response.status, response.reason, body)
-
-    # def enable_logging(self, target_bucket, target_prefix='',
-    #                    grants=None, headers=None):
-    #     """
-    #     Enable logging on a bucket.
-    #
-    #     :type target_bucket: bucket or string
-    #     :param target_bucket: The bucket to log to.
-    #
-    #     :type target_prefix: string
-    #     :param target_prefix: The prefix which should be prepended to the
-    #         generated log files written to the target_bucket.
-    #
-    #     :type grants: list of Grant objects
-    #     :param grants: A list of extra permissions which will be granted on
-    #         the log files which are created.
-    #
-    #     :rtype: bool
-    #     :return: True if ok or raises an exception.
-    #     """
-    #     if isinstance(target_bucket, Bucket):
-    #         target_bucket = target_bucket.name
-    #     blogging = BucketLogging(target=target_bucket, prefix=target_prefix,
-    #                              grants=grants)
-    #     return self.set_xml_logging(blogging.to_xml(), headers=headers)
-    #
-    # def disable_logging(self, headers=None):
-    #     """
-    #     Disable logging on a bucket.
-    #
-    #     :rtype: bool
-    #     :return: True if ok or raises an exception.
-    #     """
-    #     blogging = BucketLogging()
-    #     return self.set_xml_logging(blogging.to_xml(), headers=headers)
-
-    # def get_logging_status(self, headers=None):
-    #     """
-    #     Get the logging status for this bucket.
-    #
-    #     :rtype: :class:`boto.s3.bucketlogging.BucketLogging`
-    #     :return: A BucketLogging object for this bucket.
-    #     """
-    #     response = self.connection.make_request('GET', self.name,
-    #             query_args={'logging':''}, headers=headers)
-    #     body = response.read()
-    #     if response.status == 200:
-    #         blogging = BucketLogging()
-    #         h = handler.XmlHandler(blogging, self)
-    #         if not isinstance(body, bytes):
-    #             body = body.encode('utf-8')
-    #         xml.sax.parseString(body, h)
-    #         return blogging
-    #     else:
-    #         raise S3ResponseError(response.status, response.reason, body)
-
     def generate_url(self, expires_in, method, bucket='', key='', headers=None,
                      query_auth=True, force_http=False, response_headers=None,
                      expires_in_absolute=False, version_id=None):
@@ -343,7 +266,7 @@ class Connection(object):
             delimiter = '?' if '?' not in auth_path else '&'
             auth_path += delimiter + '&'.join(extra_qp)
 
-        #if not headers.has_key('Date'):
+        # if not headers.has_key('Date'):
         #    headers['Date'] = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())
 
         c_string = auth.canonical_string(method, bucket, key, headers=headers, expires=expires)
@@ -352,8 +275,8 @@ class Connection(object):
         self.calling_format.build_path_base(bucket, key)
         if query_auth:
             encode_ak = self.access_key_id
-            #encode_ak = parse.quote(self.access_key_id)
-            #print 'encode_ak:%s'%encode_ak
+            # encode_ak = parse.quote(self.access_key_id)
+            # print 'encode_ak:%s'%encode_ak
             query_part = '?' + self.QueryString % (encoded_canonical, expires, encode_ak)
         else:
             query_part = ''
@@ -411,3 +334,10 @@ class Connection(object):
             path = path + params
         return path
 
+    def get_adp(self, task_id):
+        query_args = 'queryadp'
+        response = self.make_request('GET', task_id, query_args=query_args)
+        body = response.read()
+        if response.status != 200:
+            raise S3ResponseError(response.status, response.reason, body)
+        return body
